@@ -13,10 +13,9 @@
 #include "oxygenshadowcache.h"
 
 #include "oxygenbutton.h"
-#include "oxygensizegrip.h"
 
-#include <KDecoration2/DecorationButtonGroup>
-#include <KDecoration2/DecorationShadow>
+#include <KDecoration3/DecorationButtonGroup>
+#include <KDecoration3/DecorationShadow>
 
 #include <KColorUtils>
 #include <KConfigGroup>
@@ -35,17 +34,17 @@ K_PLUGIN_FACTORY_WITH_JSON(OxygenDecoFactory, "oxygen.json", registerPlugin<Oxyg
 
 namespace Oxygen
 {
-using KDecoration2::ColorGroup;
-using KDecoration2::ColorRole;
+using KDecoration3::ColorGroup;
+using KDecoration3::ColorRole;
 
 //________________________________________________________________
-using ShadowMap = QHash<int, std::shared_ptr<KDecoration2::DecorationShadow>>;
+using ShadowMap = QHash<int, std::shared_ptr<KDecoration3::DecorationShadow>>;
 
 static int g_sDecoCount = 0;
 static ShadowMap g_sShadows;
 
 Decoration::Decoration(QObject *parent, const QVariantList &args)
-    : KDecoration2::Decoration(parent, args)
+    : KDecoration3::Decoration(parent, args)
     , m_animation(new QPropertyAnimation(this))
 {
     g_sDecoCount++;
@@ -57,8 +56,6 @@ Decoration::~Decoration()
     g_sDecoCount--;
     if (g_sDecoCount == 0)
         g_sShadows.clear();
-
-    deleteSizeGrip();
 }
 
 //________________________________________________________________
@@ -69,9 +66,6 @@ void Decoration::setOpacity(qreal value)
     m_opacity = value;
     updateShadow();
     update();
-
-    if (m_sizeGrip)
-        m_sizeGrip->update();
 }
 
 //_________________________________________________________
@@ -81,7 +75,7 @@ QColor Decoration::titleBarColor(const QPalette &palette) const
         return KColorUtils::mix(titleBarColor(palette, false), titleBarColor(palette, true), m_opacity);
 
     } else {
-        return titleBarColor(palette, client()->isActive());
+        return titleBarColor(palette, window()->isActive());
     }
 }
 
@@ -92,7 +86,7 @@ QColor Decoration::titleBarColor(const QPalette &palette, bool active) const
         return palette.color(active ? QPalette::Active : QPalette::Inactive, QPalette::Window);
 
     } else {
-        return client()->color(active ? ColorGroup::Active : ColorGroup::Inactive, ColorRole::TitleBar);
+        return window()->color(active ? ColorGroup::Active : ColorGroup::Inactive, ColorRole::TitleBar);
     }
 }
 
@@ -105,7 +99,7 @@ QColor Decoration::fontColor(const QPalette &palette) const
         return KColorUtils::mix(fontColor(palette, false), fontColor(palette, true), m_opacity);
 
     } else {
-        return fontColor(palette, client()->isActive());
+        return fontColor(palette, window()->isActive());
     }
 }
 
@@ -116,7 +110,7 @@ QColor Decoration::fontColor(const QPalette &palette, bool active) const
         return palette.color(active ? QPalette::Active : QPalette::Disabled, QPalette::WindowText);
 
     } else {
-        return client()->color(active ? ColorGroup::Active : ColorGroup::Inactive, ColorRole::Foreground);
+        return window()->color(active ? ColorGroup::Active : ColorGroup::Inactive, ColorRole::Foreground);
     }
 }
 
@@ -126,7 +120,7 @@ QColor Decoration::contrastColor(const QPalette &palette) const
     if (m_internalSettings->useWindowColors())
         return contrastColor(palette.color(QPalette::Window));
     else {
-        const auto cl = client();
+        const auto cl = window();
         return contrastColor(cl->color(cl->isActive() ? ColorGroup::Active : ColorGroup::Inactive, ColorRole::TitleBar));
     }
 }
@@ -150,43 +144,43 @@ bool Decoration::init()
     reconfigure();
     updateTitleBar();
     auto s = settings();
-    connect(s.get(), &KDecoration2::DecorationSettings::borderSizeChanged, this, &Decoration::recalculateBorders);
+    connect(s.get(), &KDecoration3::DecorationSettings::borderSizeChanged, this, &Decoration::recalculateBorders);
 
     // a change in font might cause the borders to change
-    connect(s.get(), &KDecoration2::DecorationSettings::fontChanged, this, &Decoration::recalculateBorders);
-    connect(s.get(), &KDecoration2::DecorationSettings::spacingChanged, this, &Decoration::recalculateBorders);
+    connect(s.get(), &KDecoration3::DecorationSettings::fontChanged, this, &Decoration::recalculateBorders);
+    connect(s.get(), &KDecoration3::DecorationSettings::spacingChanged, this, &Decoration::recalculateBorders);
 
     // buttons
-    connect(s.get(), &KDecoration2::DecorationSettings::spacingChanged, this, &Decoration::updateButtonsGeometryDelayed);
-    connect(s.get(), &KDecoration2::DecorationSettings::decorationButtonsLeftChanged, this, &Decoration::updateButtonsGeometryDelayed);
-    connect(s.get(), &KDecoration2::DecorationSettings::decorationButtonsRightChanged, this, &Decoration::updateButtonsGeometryDelayed);
+    connect(s.get(), &KDecoration3::DecorationSettings::spacingChanged, this, &Decoration::updateButtonsGeometryDelayed);
+    connect(s.get(), &KDecoration3::DecorationSettings::decorationButtonsLeftChanged, this, &Decoration::updateButtonsGeometryDelayed);
+    connect(s.get(), &KDecoration3::DecorationSettings::decorationButtonsRightChanged, this, &Decoration::updateButtonsGeometryDelayed);
 
     // full reconfiguration
-    connect(s.get(), &KDecoration2::DecorationSettings::reconfigured, this, &Decoration::reconfigure);
-    connect(s.get(), &KDecoration2::DecorationSettings::reconfigured, SettingsProvider::self(), &SettingsProvider::reconfigure, Qt::UniqueConnection);
+    connect(s.get(), &KDecoration3::DecorationSettings::reconfigured, this, &Decoration::reconfigure);
+    connect(s.get(), &KDecoration3::DecorationSettings::reconfigured, SettingsProvider::self(), &SettingsProvider::reconfigure, Qt::UniqueConnection);
 
-    const auto *cl = client();
-    connect(cl, &KDecoration2::DecoratedClient::adjacentScreenEdgesChanged, this, &Decoration::recalculateBorders);
-    connect(cl, &KDecoration2::DecoratedClient::maximizedHorizontallyChanged, this, &Decoration::recalculateBorders);
-    connect(cl, &KDecoration2::DecoratedClient::maximizedVerticallyChanged, this, &Decoration::recalculateBorders);
-    connect(cl, &KDecoration2::DecoratedClient::captionChanged, this, [this]() {
+    const auto *cl = window();
+    connect(cl, &KDecoration3::DecoratedWindow::adjacentScreenEdgesChanged, this, &Decoration::recalculateBorders);
+    connect(cl, &KDecoration3::DecoratedWindow::maximizedHorizontallyChanged, this, &Decoration::recalculateBorders);
+    connect(cl, &KDecoration3::DecoratedWindow::maximizedVerticallyChanged, this, &Decoration::recalculateBorders);
+    connect(cl, &KDecoration3::DecoratedWindow::captionChanged, this, [this]() {
         // update the caption area
         update(titleBar());
     });
 
-    connect(cl, &KDecoration2::DecoratedClient::activeChanged, this, &Decoration::updateAnimationState);
-    connect(cl, &KDecoration2::DecoratedClient::activeChanged, this, &Decoration::updateShadow);
+    connect(cl, &KDecoration3::DecoratedWindow::activeChanged, this, &Decoration::updateAnimationState);
+    connect(cl, &KDecoration3::DecoratedWindow::activeChanged, this, &Decoration::updateShadow);
 
     // decoration has an overloaded update function, force the compiler to choose the right one
-    connect(cl, &KDecoration2::DecoratedClient::paletteChanged, this, static_cast<void (Decoration::*)()>(&Decoration::update));
-    connect(cl, &KDecoration2::DecoratedClient::widthChanged, this, &Decoration::updateTitleBar);
-    connect(cl, &KDecoration2::DecoratedClient::maximizedChanged, this, &Decoration::updateTitleBar);
-    connect(cl, &KDecoration2::DecoratedClient::maximizedChanged, this, &Decoration::setOpaque);
+    connect(cl, &KDecoration3::DecoratedWindow::paletteChanged, this, static_cast<void (Decoration::*)()>(&Decoration::update));
+    connect(cl, &KDecoration3::DecoratedWindow::widthChanged, this, &Decoration::updateTitleBar);
+    connect(cl, &KDecoration3::DecoratedWindow::maximizedChanged, this, &Decoration::updateTitleBar);
+    connect(cl, &KDecoration3::DecoratedWindow::maximizedChanged, this, &Decoration::setOpaque);
 
-    connect(cl, &KDecoration2::DecoratedClient::widthChanged, this, &Decoration::updateButtonsGeometry);
-    connect(cl, &KDecoration2::DecoratedClient::maximizedChanged, this, &Decoration::updateButtonsGeometry);
-    connect(cl, &KDecoration2::DecoratedClient::shadedChanged, this, &Decoration::recalculateBorders);
-    connect(cl, &KDecoration2::DecoratedClient::shadedChanged, this, &Decoration::updateButtonsGeometry);
+    connect(cl, &KDecoration3::DecoratedWindow::widthChanged, this, &Decoration::updateButtonsGeometry);
+    connect(cl, &KDecoration3::DecoratedWindow::maximizedChanged, this, &Decoration::updateButtonsGeometry);
+    connect(cl, &KDecoration3::DecoratedWindow::shadedChanged, this, &Decoration::recalculateBorders);
+    connect(cl, &KDecoration3::DecoratedWindow::shadedChanged, this, &Decoration::updateButtonsGeometry);
 
     createButtons();
     updateShadow();
@@ -198,7 +192,7 @@ void Decoration::updateTitleBar()
 {
     auto s = settings();
     const bool maximized = isMaximized();
-    const auto clientPtr = client();
+    const auto clientPtr = window();
     const int width = maximized ? clientPtr->width() : clientPtr->width() - 2 * s->largeSpacing() * Metrics::TitleBar_SideMargin;
     const int height = maximized ? borderTop() : borderTop() - s->smallSpacing() * Metrics::TitleBar_TopMargin;
     const int x = maximized ? 0 : s->largeSpacing() * Metrics::TitleBar_SideMargin;
@@ -210,7 +204,7 @@ void Decoration::updateTitleBar()
 void Decoration::updateAnimationState()
 {
     if (m_internalSettings->animationsEnabled()) {
-        m_animation->setDirection(client()->isActive() ? QPropertyAnimation::Forward : QPropertyAnimation::Backward);
+        m_animation->setDirection(window()->isActive() ? QPropertyAnimation::Forward : QPropertyAnimation::Backward);
         if (m_animation->state() != QPropertyAnimation::Running)
             m_animation->start();
 
@@ -219,15 +213,6 @@ void Decoration::updateAnimationState()
     }
 }
 
-//________________________________________________________________
-void Decoration::updateSizeGripVisibility()
-{
-    const auto c = client();
-    if (m_sizeGrip) {
-        m_sizeGrip->setVisible(c->isResizeable() && !isMaximized() && !c->isShaded());
-    }
-}
-
 //________________________________________________________________
 int Decoration::borderSize(bool bottom) const
 {
@@ -257,24 +242,24 @@ int Decoration::borderSize(bool bottom) const
 
     } else {
         switch (settings()->borderSize()) {
-        case KDecoration2::BorderSize::None:
+        case KDecoration3::BorderSize::None:
             return 0;
-        case KDecoration2::BorderSize::NoSides:
+        case KDecoration3::BorderSize::NoSides:
             return bottom ? qMax(4, baseSize) : 0;
         default:
-        case KDecoration2::BorderSize::Tiny:
+        case KDecoration3::BorderSize::Tiny:
             return bottom ? qMax(4, baseSize) : baseSize;
-        case KDecoration2::BorderSize::Normal:
+        case KDecoration3::BorderSize::Normal:
             return baseSize * 2;
-        case KDecoration2::BorderSize::Large:
+        case KDecoration3::BorderSize::Large:
             return baseSize * 3;
-        case KDecoration2::BorderSize::VeryLarge:
+        case KDecoration3::BorderSize::VeryLarge:
             return baseSize * 4;
-        case KDecoration2::BorderSize::Huge:
+        case KDecoration3::BorderSize::Huge:
             return baseSize * 5;
-        case KDecoration2::BorderSize::VeryHuge:
+        case KDecoration3::BorderSize::VeryHuge:
             return baseSize * 6;
-        case KDecoration2::BorderSize::Oversized:
+        case KDecoration3::BorderSize::Oversized:
             return baseSize * 10;
         }
     }
@@ -293,19 +278,13 @@ void Decoration::reconfigure()
 
     // clear shadows
     g_sShadows.clear();
-
-    // size grip
-    if (hasNoBorders() && m_internalSettings->drawSizeGrip())
-        createSizeGrip();
-    else
-        deleteSizeGrip();
 }
 
 //________________________________________________________________
 void Decoration::recalculateBorders()
 {
     auto s = settings();
-    const auto c = client();
+    const auto c = window();
     const auto edges = c->adjacentScreenEdges();
 
     // left, right and bottom borders
@@ -353,8 +332,8 @@ void Decoration::recalculateBorders()
 //________________________________________________________________
 void Decoration::createButtons()
 {
-    m_leftButtons = new KDecoration2::DecorationButtonGroup(KDecoration2::DecorationButtonGroup::Position::Left, this, &Button::create);
-    m_rightButtons = new KDecoration2::DecorationButtonGroup(KDecoration2::DecorationButtonGroup::Position::Right, this, &Button::create);
+    m_leftButtons = new KDecoration3::DecorationButtonGroup(KDecoration3::DecorationButtonGroup::Position::Left, this, &Button::create);
+    m_rightButtons = new KDecoration3::DecorationButtonGroup(KDecoration3::DecorationButtonGroup::Position::Right, this, &Button::create);
     updateButtonsGeometry();
 }
 
@@ -374,8 +353,8 @@ void Decoration::updateButtonsGeometry()
     const int bWidth = buttonHeight();
     const int verticalOffset = (isMaximized() ? s->smallSpacing() * Metrics::TitleBar_TopMargin : 0) + (captionHeight() - buttonHeight()) / 2;
 
-    const QVector<KDecoration2::DecorationButton *> leftButtons = m_leftButtons->buttons();
-    const QVector<KDecoration2::DecorationButton *> rightButtons = m_rightButtons->buttons();
+    const QVector<KDecoration3::DecorationButton *> leftButtons = m_leftButtons->buttons();
+    const QVector<KDecoration3::DecorationButton *> rightButtons = m_rightButtons->buttons();
 
     const auto allButtons = leftButtons + rightButtons;
     for (const auto &button : allButtons) {
@@ -428,9 +407,9 @@ void Decoration::updateButtonsGeometry()
 }
 
 //________________________________________________________________
-void Decoration::paint(QPainter *painter, const QRect &repaintRegion)
+void Decoration::paint(QPainter *painter, const QRectF &repaintRegion)
 {
-    const auto c = client();
+    const auto c = window();
     const auto palette = c->palette();
 
     const auto rect = c->isShaded() ? QRect(QPoint(0, 0), QSize(size().width(), borderTop())) : this->rect();
@@ -500,7 +479,7 @@ QPair<QRect, Qt::Alignment> Decoration::captionRect() const
         case InternalSettings::AlignCenterFullWidth: {
             // full caption rect
             const QRect fullRect = QRect(0, yOffset, size().width(), captionHeight());
-            QRect boundingRect(settings()->fontMetrics().boundingRect(client()->caption()).toRect());
+            QRect boundingRect(settings()->fontMetrics().boundingRect(window()->caption()).toRect());
 
             // text bounding rect
             boundingRect.setTop(yOffset);
@@ -532,7 +511,7 @@ void Decoration::updateShadow()
 
     // generate key
     ShadowCache::Key key;
-    const auto clientPtr = client();
+    const auto clientPtr = window();
     key.active = SettingsProvider::self()->shadowCache()->isEnabled(QPalette::Active) && clientPtr->isActive();
     key.isShade = clientPtr->isShaded();
     key.hasBorder = !hasNoBorders();
@@ -547,7 +526,7 @@ void Decoration::updateShadow()
     // find key in map
     auto iter = g_sShadows.find(hash);
     if (iter == g_sShadows.end()) {
-        auto decorationShadow = std::make_shared<KDecoration2::DecorationShadow>();
+        auto decorationShadow = std::make_shared<KDecoration3::DecorationShadow>();
         QPixmap shadowPixmap =
             animated ? SettingsProvider::self()->shadowCache()->animatedPixmap(key, m_opacity) : SettingsProvider::self()->shadowCache()->pixmap(key);
 
@@ -564,24 +543,24 @@ void Decoration::updateShadow()
 }
 
 //_________________________________________________________
-void Decoration::renderCorners(QPainter *painter, const QRect &frame, const QPalette &palette) const
+void Decoration::renderCorners(QPainter *painter, const QRectF &frame, const QPalette &palette) const
 {
     const QColor color(titleBarColor(palette));
 
-    QLinearGradient lg = QLinearGradient(0, -0.5, 0, qreal(frame.height()) + 0.5);
+    QLinearGradient lg = QLinearGradient(0, -0.5, 0, frame.height() + 0.5);
     lg.setColorAt(0.0, SettingsProvider::self()->helper()->calcLightColor(SettingsProvider::self()->helper()->backgroundTopColor(color)));
     lg.setColorAt(0.51, SettingsProvider::self()->helper()->backgroundBottomColor(color));
     lg.setColorAt(1.0, SettingsProvider::self()->helper()->backgroundBottomColor(color));
 
     painter->setPen(QPen(lg, 1));
     painter->setBrush(Qt::NoBrush);
-    painter->drawRoundedRect(QRectF(frame).adjusted(0.5, 0.5, -0.5, -0.5), 3.5, 3.5);
+    painter->drawRoundedRect(frame.adjusted(0.5, 0.5, -0.5, -0.5), 3.5, 3.5);
 }
 
 //_________________________________________________________
-void Decoration::renderWindowBackground(QPainter *painter, const QRect &clipRect, const QPalette &palette) const
+void Decoration::renderWindowBackground(QPainter *painter, const QRectF &clipRect, const QPalette &palette) const
 {
-    const auto c = client();
+    const auto c = window();
     auto innerClientRect = c->isShaded() ? QRect(QPoint(0, 0), QSize(size().width(), borderTop())) : rect();
 
     // size of window minus the outlines for the rounded corners
@@ -589,21 +568,16 @@ void Decoration::renderWindowBackground(QPainter *painter, const QRect &clipRect
         innerClientRect.adjust(1, 1, -1, -1);
     }
 
-    if (SettingsProvider::self()->helper()->hasBackgroundGradient(c->windowId()) || !SettingsProvider::self()->helper()->isX11()) {
-        SettingsProvider::self()->helper()->renderWindowBackground(painter, clipRect, innerClientRect, titleBarColor(palette), borderTop() - 24);
-
-    } else {
-        painter->fillRect(innerClientRect, titleBarColor(palette));
-    }
+    SettingsProvider::self()->helper()->renderWindowBackground(painter, clipRect, innerClientRect, titleBarColor(palette), borderTop() - 24);
 }
 
 //_________________________________________________________
-void Decoration::renderWindowBorder(QPainter *painter, const QRect &clipRect, const QPalette &palette) const
+void Decoration::renderWindowBorder(QPainter *painter, const QRectF &clipRect, const QPalette &palette) const
 {
     // save painter
     if (clipRect.isValid()) {
         painter->save();
-        painter->setClipRegion(clipRect, Qt::IntersectClip);
+        painter->setClipRect(clipRect, Qt::IntersectClip);
     }
 
     // title height
@@ -625,7 +599,7 @@ void Decoration::renderTitleText(QPainter *painter, const QPalette &palette) con
     const auto cR = captionRect();
 
     // copy caption
-    const QString caption = painter->fontMetrics().elidedText(client()->caption(), Qt::ElideMiddle, cR.first.width());
+    const QString caption = painter->fontMetrics().elidedText(window()->caption(), Qt::ElideMiddle, cR.first.width());
 
     const auto contrast(contrastColor(palette));
     if (contrast.isValid()) {
@@ -640,35 +614,6 @@ void Decoration::renderTitleText(QPainter *painter, const QPalette &palette) con
     painter->drawText(cR.first, cR.second | Qt::TextSingleLine, caption);
 }
 
-//_________________________________________________________________
-void Decoration::createSizeGrip(void)
-{
-    // do nothing if size grip already exist
-    if (m_sizeGrip)
-        return;
-
-#if OXYGEN_HAVE_X11
-    if (!QX11Info::isPlatformX11())
-        return;
-
-    if (client()->windowId() != 0) {
-        m_sizeGrip = new SizeGrip(this);
-        connect(client(), &KDecoration2::DecoratedClient::maximizedChanged, this, &Decoration::updateSizeGripVisibility);
-        connect(client(), &KDecoration2::DecoratedClient::shadedChanged, this, &Decoration::updateSizeGripVisibility);
-        connect(client(), &KDecoration2::DecoratedClient::resizeableChanged, this, &Decoration::updateSizeGripVisibility);
-    }
-#endif
-}
-
-//_________________________________________________________________
-void Decoration::deleteSizeGrip(void)
-{
-    if (m_sizeGrip) {
-        m_sizeGrip->deleteLater();
-        m_sizeGrip = nullptr;
-    }
-}
-
 } // namespace
 
 #include "oxygendecoration.moc"
